package org.gemoc.executionframework.engine.core;

import com.google.common.base.Objects;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.transaction.RollbackException;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.gemoc.executionframework.engine.mse.MSE;
import org.gemoc.executionframework.engine.mse.MSEOccurrence;

/**
 * An exception that is caused by anything thrown from the execution of
 * sequential operational semantics.
 */
@SuppressWarnings("all")
public class SequentialExecutionException extends RuntimeException {
  private MSEOccurrence pendingMSEOccurrence;
  
  public SequentialExecutionException(final MSEOccurrence pendingMSE, final Throwable cause) {
    this.pendingMSEOccurrence = pendingMSE;
    this.initCause(cause);
  }
  
  private String prettyPrintMSEOcc() {
    boolean _notEquals = (!Objects.equal(this.pendingMSEOccurrence, null));
    if (_notEquals) {
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("Pending MSEOccurrence: ");
      MSE _mse = this.pendingMSEOccurrence.getMse();
      EObject _caller = _mse.getCaller();
      EClass _eClass = _caller.eClass();
      String _name = _eClass.getName();
      _builder.append(_name, "");
      _builder.append(".");
      MSE _mse_1 = this.pendingMSEOccurrence.getMse();
      EOperation _action = _mse_1.getAction();
      String _name_1 = _action.getName();
      _builder.append(_name_1, "");
      _builder.append(" called on ");
      MSE _mse_2 = this.pendingMSEOccurrence.getMse();
      EObject _caller_1 = _mse_2.getCaller();
      _builder.append(_caller_1, "");
      _builder.append(".");
      return _builder.toString();
    } else {
      return "No pending MSE.";
    }
  }
  
  @Override
  public String getMessage() {
    boolean _and = false;
    Throwable _cause = this.getCause();
    boolean _notEquals = (!Objects.equal(_cause, null));
    if (!_notEquals) {
      _and = false;
    } else {
      Throwable _cause_1 = this.getCause();
      _and = (_cause_1 instanceof RollbackException);
    }
    if (_and) {
      String _prettyPrintMSEOcc = this.prettyPrintMSEOcc();
      return ("An error occured during the execution of the operational semantics (originally catched as a RollbackException during the transaction commit).\n" + _prettyPrintMSEOcc);
    } else {
      boolean _and_1 = false;
      Throwable _cause_2 = this.getCause();
      boolean _notEquals_1 = (!Objects.equal(_cause_2, null));
      if (!_notEquals_1) {
        _and_1 = false;
      } else {
        Throwable _cause_3 = this.getCause();
        _and_1 = (_cause_3 instanceof InterruptedException);
      }
      if (_and_1) {
        String _prettyPrintMSEOcc_1 = this.prettyPrintMSEOcc();
        return ("The engine thread was interrupted while it was waiting for being allowed to start an execution step\'s transaction.\n" + _prettyPrintMSEOcc_1);
      } else {
        return super.getMessage();
      }
    }
  }
}
